/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : MARCH 2009

 File          : ViewADI_OSC_REF.c

 Hardware      : Applicable to ADuC7038

 Description   : This routine Sets the ECLK on GPIO_5.

	Oscillator	 This clock is 2.56MHZ nominal - 20 x the 128KHz oscillator
				 The user can vary	POWCON to set the Low power Oscillator - 3%
				 or the Precision oscillator - 1%  , measure the 2.56MHZ clock 
				 and Divide by 20 for 128KHz measurement. 

	Reference:	 Also , This routine sets up the ADC to use the current channel 
				 to measure internal reference and expects a good quality input 
				 Voltage at IIN+ and IIN-. 
				 VBAT can also be used  - code added 

*********************************************************************/

#include <ADuC7038.h>


unsigned int VBAT_data=0;
	 float Vbat_approx=0;

	 float VRef_data=0;
	 float Verror = 0.0;

unsigned int IADC_data=0;
	float IADC_approx=0;

unsigned char adc_status=0;

void ADC_VBAT_SetUp(void);
void delay(int);

	unsigned int Any_status	= 0;

int main()
{
	unsigned long it1=1234, it2=4321;
									// SET 10.24 MEg PLL
 	POWKEY0 = 0x01;		 			// Put all peripherals, PLL and Core Asleep
	POWCON	 = 0x179;				// 0001 0111 1001 default value is 79	- 10MHz	/ 1% prec
	POWKEY1 = 0xF4;					// 
	it1 = it1*it2;					// Dummy cycle to ensure correct power down 

    delay(20000);					// Delay after enabling 1% osc - PLLCON

  PLLKEY0 = 0x000000AA;
	PLLCON  = 0x01;        	 /* Intern 1% Oszillator  Precision   */
  PLLKEY1 = 0x00000055;
	it1 = it1*it2;			// Dummy cycle to ensure correct power down

	Any_status = PLLSTA;	// check Lock
	do 
		{
		Any_status = PLLSTA;
		}
	while ((Any_status & 0x02) != 2);

  	GPCON = 0x00200000;				//Enable (GPIO 5 as ECLK) and all others as ouputs.
									// View on Scope  - Divide by 20 for precision Measurment on 128 KHz clock
	GPDAT = 0x3F000000;				
 	GPSET = 0x00030000;				// View Pulse on Scope - ensures that PLL Lock occurred !
	GPCLR = 0x00030000;				// Set all bits to Zero

 	GPSET = 0x00030000;				// View Pulse on Scope - ensures that VBAT Voltage is read correctly!
	GPCLR = 0x00030000;				// Set all bits to Zero

	while(1)				
	{
 	}    							// Infinite Loop
}

void delay (int length){
	while (length >=0)
    	length--;
}

void ADC_VBAT_SetUp(void)		 // Step through this.
{

	 ADCMSKI = 0x00;		// interupt Mask  - No need to do this but ...
	 ADCMDE = 0x01;		// 0000 0001    NO int, Normal mode, Cont conv.
	 ADC1CON = 0x8200;	    // ADC enabld, uA OFF, uni, VBAT
	 ADC0CON = 0x8202;		// 1000 0010 0010 0000  ADC enabld, uA OFF, uni, VBAT, 1.2V Ref , Gain 1 	
	 ADCFLT	= 0xC07F;		// ChopEn, Run Av, AF n/A , SINC mod, DEC Fac 50Hz notch
	 						//  1100 0000 0111 1111
 	 ADCCFG = 0x00;			//  No Acc, NO Fast Temp, No Counter
	 						// 0000 0000
	 while((ADCSTA & 0x02) == 1);
 	 VBAT_data = ADC1DAT;	// Dummy data read
	do 
	 {
	  VBAT_data = ADC1DAT;		// Check VBAT for Sanity
	  Vbat_approx = VBAT_data;
	  Vbat_approx = Vbat_approx * 1.2 * 24;
	  Vbat_approx = Vbat_approx / 65536;		 // approx VBAT voltage
	 }
	 while(Vbat_approx < 11.8);		// Set the Supply to 12.000V minimum For this to Pass.

	 			// Zero Scale Calibration
	 ADC1CON = 0x0200;	    // ADC disabled, uA OFF, uni, VBAT
	 ADCMDE = 0x06; 		// Zero Scale calibration
	 while (( ADCSTA & 0x80) == 0x80); //Wait for calibration to complete
	 ADC0OF = ADC0DAT; 		 // Write data	- OFFSET

	 ADCMDE = 0x07; 		// Full Scale calibration
	 while (( ADCSTA & 0x80) == 0x80); //Wait for calibration to complete
	 ADC0GN = ADC0DAT; 		 // Write data - GAIN
	  
 	 ADC1CON = 0x8200;	    // ADC denabled, uA OFF, uni, VBAT
  
 }


